from collections import deque

def find_guarded(n, m, roads, guards):
    adjacency_list = [[] for _ in range(n + 1)]
    for x, y in roads:
        adjacency_list[x].append(y)
        adjacency_list[y].append(x)

    max_range = [-1] * (n + 1)

    q = deque()
    for start, range_limit in guards:
        if range_limit > max_range[start]:
            max_range[start] = range_limit
            q.append((start, range_limit))

    while q:
        current_node, remaining_range = q.popleft()

        if remaining_range == 0:
            continue

        for neighbor in adjacency_list[current_node]:
            new_range = remaining_range - 1
            if new_range > max_range[neighbor]:
                max_range[neighbor] = new_range
                q.append((neighbor, new_range))

    secured_nodes = [node for node in range(1, n + 1) if max_range[node] >= 0]

    return secured_nodes

def main():
    import sys
    import sys
    try:
        n, m, k = map(int, input().split())
    except ValueError:
        n = m = k = 0

    roads = []
    for _ in range(m):
        try:
            x, y = map(int, input().split())
            roads.append((x, y))
        except ValueError:
            continue

    guards = []
    for _ in range(k):
        try:
            start, range_limit = map(int, input().split())
            guards.append((start, range_limit))
        except ValueError:
            continue

    secured = find_guarded(n, m, roads, guards)

    print(len(secured))
    print(' '.join(map(str, sorted(secured))))

if __name__ == "__main__":
    main()
